%{
This is a supplementary file for the paper of
Andrey Kan, Rajib Chakravorty, James Bailey,
Christopher Leckie, John Markham, Mark R. Dowling.

Andrey Kan (akan@csse.unimelb.edu.au) is the corresponding author.

Do not redistribute, do not use for commercial purposes.

Property of National ICT Australia.
=====================================

Method for Reconstruction of the Distribution of Cell Moves (MRD)

The method allows to estimate CDF of true links
(and hence the average and maximum speeds of cells)
without the need for the ground truth (GT) for a video.

MRD can be used to estimate the optimal gating distance

detResults -- detection results, see "sampleDet.mat" for data format
Pfp, Pfn -- probabilities of false positive and false negative errors, can be omitted

optGD -- estimated optimal gating distance
%}
function optGD = analyseMovieMRD(detResults, Pfp, Pfn)

%% default vals
if (nargin < 3)
	Pfp = 0;
	Pfn = 0;
end
nFrames = size(detResults, 2);

%% false links = intraframe (spatial) associations
rf = [];
nCells = zeros(1, nFrames);
for frameIdx = 1:nFrames
	locsCurr = detResults(frameIdx).centroid;
	sizeCurr = size(locsCurr, 1);
	nCells(frameIdx) = sizeCurr;
	
	% distances from each cell to each cell
	for idx1 = 1:sizeCurr
		loc1 = locsCurr(idx1,:);
		for idx2 = (idx1 + 1):sizeCurr
			loc2 = locsCurr(idx2,:);
			dx = loc1(1,1) - loc2(1,1);
			dy = loc1(1,2) - loc2(1,2);
			r = sqrt(dx^2 + dy^2);
			rf = [rf, r]; %#ok<AGROW>
		end
	end
end

%% all links = interframe (temporal) associations
ra = []; % all links
locsPrev = detResults(1).centroid;
sizePrev = size(locsPrev, 1);
for frameIdx = 2:nFrames
	locsNext = detResults(frameIdx).centroid;
	sizeNext = size(locsNext, 1);
	
	% from each cell at the previous frame
	for idxPrev = 1:sizePrev
		locPrev = locsPrev(idxPrev,:);
		
		% to each cell at the next frame
		for idxNext = 1:sizeNext
			locNext = locsNext(idxNext,:);
			
			dx = locPrev(1,1) - locNext(1,1);
			dy = locPrev(1,2) - locNext(1,2);
			r = sqrt(dx^2 + dy^2);
			ra = [ra, r]; %#ok<AGROW>
		end
	end
	locsPrev = locsNext;
	sizePrev = sizeNext;
end

%% true links <= estimation of CDF
vidProps.raMax = max(ra);
na = size(ra, 2);
nt = (1 - Pfp)*(1 - Pfn)*sum(nCells(2:end));
nf = na - nt;

[ya, xa] = ecdf(ra);
[yf, xf] = ecdf(rf);

xa(1) = -0.0001;
xf(1) = -0.0001;

rmax = vidProps.raMax + 1;
r = 1:rmax;

pa = interp1(xa, ya, r, 'linear', 1);
pf = interp1(xf, yf, r, 'linear', 1);

pt = (pa .* na - pf .* nf) ./ nt;

%% make the CDF smoother <= median filtering
[pks lmax] = findpeaks(pt);
[pks lmin] = findpeaks(-pt);
lpks = sort([lmax lmin]);
if (size(lpks, 2) > 1)
	wnd = 2*max(lpks(2:end) - lpks(1:(end-1)));
	%stub = zeros(1, wnd);
	%stub(:) = pt(1);
	%pt = [stub pt];
	pt = medfilt1(pt, wnd);
	%pt(1:wnd) = [];
end


%% estimate the optimal gating distance
maxval = max(pt);
if (maxval <= 0.97)
	error('estimation of CDF is very inaccurate');
end
t = min(maxval, 1);
optGD = find(pt >= t, 1);


end